"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _DatabaseWrapper_database;
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseWrapper = exports.DatabaseManager = void 0;
const settings_1 = require("./settings");
const core_1 = require("@rtdb2/core");
const rules_1 = require("@rtdb2/rules");
const logging_1 = require("@hibas123/logging");
class DatabaseManager {
    static async init() {
        let databases = await settings_1.default.getDatabases();
        databases.forEach((dbconfig) => {
            let db = new DatabaseWrapper(dbconfig.name, dbconfig.accesskey, dbconfig.rules, dbconfig.publickey, dbconfig.rootkey);
            this.databases.set(dbconfig.name, db);
        });
    }
    static async addDatabase(name) {
        if (this.databases.has(name))
            throw new Error("Database already exists!");
        await settings_1.default.addDatabase(name);
        let database = new DatabaseWrapper(name);
        this.databases.set(name, database);
        return database;
    }
    static getDatabase(name) {
        return this.databases.get(name);
    }
    static async deleteDatabase(name) {
        let db = this.databases.get(name);
        if (db) {
            await settings_1.default.deleteDatabase(name);
            await db.database.delete();
        }
    }
}
exports.DatabaseManager = DatabaseManager;
DatabaseManager.databases = new Map();
class DatabaseWrapper {
    constructor(name, accesskey, rules, publickey, rootkey) {
        this.name = name;
        this.accesskey = accesskey;
        this.rules = rules;
        this.publickey = publickey;
        this.rootkey = rootkey;
        _DatabaseWrapper_database.set(this, void 0);
        this.rules = rules;
        __classPrivateFieldSet(this, _DatabaseWrapper_database, new core_1.Database("./databases/" + name, {
            // Set blocking rule at the start
            hasPermission: () => false,
        }), "f");
        __classPrivateFieldGet(this, _DatabaseWrapper_database, "f").logging = logging_1.default.getChild(name);
        if (rules)
            this.applyRules(rules);
    }
    get database() {
        return __classPrivateFieldGet(this, _DatabaseWrapper_database, "f");
    }
    applyRules(rules) {
        try {
            JSON.parse(rules);
            logging_1.default.warning("Found old rule! Replacing with a 100% permissive one!");
            rules =
                "service realtimedb {\n   match /* {\n      allow read, write, list: if false; \n   }\n}";
            // still json, so switching to new format
        }
        catch (err) { }
        let { runner, error } = (0, rules_1.default)(rules);
        if (error) {
            logging_1.default.warning("Found error in existing config!", error);
            runner = (0, rules_1.default)("service realtimesb {}").runner;
        }
        this.rules = rules;
        __classPrivateFieldGet(this, _DatabaseWrapper_database, "f").rules = runner;
        return undefined;
    }
    async setRules(rules) {
        const { runner, error } = (0, rules_1.default)(rules);
        if (error)
            return error;
        await settings_1.default.setDatabaseRules(this.name, rules);
        this.rules = rules;
        __classPrivateFieldGet(this, _DatabaseWrapper_database, "f").rules = runner;
    }
    async setAccessKey(key) {
        await settings_1.default.setDatabaseAccessKey(this.name, key);
        this.accesskey = key;
    }
    async setRootKey(key) {
        await settings_1.default.setDatabaseRootKey(this.name, key);
        this.rootkey = key;
    }
    async setPublicKey(key) {
        await settings_1.default.setDatabasePublicKey(this.name, key);
        this.publickey = key;
    }
}
exports.DatabaseWrapper = DatabaseWrapper;
_DatabaseWrapper_database = new WeakMap();
//# sourceMappingURL=database.js.map